package uk.ac.ed.inf.mandelbrotmaps;

import android.content.Context;
import android.preference.PreferenceManager;

public class JuliaFractalView extends AbstractFractalView{
	
	// Point paramaterising this Julia set
	private double juliaX = 0;
	private double juliaY = 0;
	
	
	
	public JuliaFractalView(Context context, FractalViewSize size) {
		super(context, size);
		
		setColouringScheme(PreferenceManager.getDefaultSharedPreferences(getContext()).getString("JULIA_COLOURS", "JuliaDefault")
				, false);

		for(int i = 0; i < noOfThreads; i++) {
			renderThreadList.get(i).setName("Julia thread " + i);
		}
		
		// Set the "maximum iteration" calculation constants
		// Empirically determined values for Julia sets.
		ITERATION_BASE = 1.58;
		ITERATION_CONSTANT_FACTOR = 6.46;
		
		// Set home area
		homeGraphArea = new MandelbrotJuliaLocation().getJuliaGraphArea();
		
		// How deep a zoom do we allow?
		MAXZOOM_LN_PIXEL = -20; // Beyond -21, "double"s break down(!).
	}
		
		
	public void setJuliaParameter(double newJuliaX, double newJuliaY) {
		//stopAllRendering();
		juliaX = newJuliaX;
		juliaY = newJuliaY;
		setGraphArea(graphArea, true);
	}
	
	
	public double[] getJuliaParam() {
		double[] juliaParam = new double[2];
		juliaParam[0] = juliaX;
		juliaParam[1] = juliaY;
		return juliaParam;
	}
	
	
	// Load a location
	void loadLocation(MandelbrotJuliaLocation mjLocation) {
		//setScaledIterationCount(mjLocation.getJuliaContrast());
		double[] juliaParam = mjLocation.getJuliaParam();
		setGraphArea(mjLocation.getJuliaGraphArea(), true);
		setJuliaParameter(juliaParam[0], juliaParam[1]);
	}
	
	
	int pixelInSet(int xPixel, int yPixel, int maxIterations, int opt) {
		boolean inside = true;
		int iterationNr;
		double newx,newy,nx,ny;
		double x,y,x0,y0;
		double a,b,es,p,q,r,rx1,ry1,rx2,ry2;
		rx1=0;
		ry1=0;
		rx2=0;
		ry2=0;
		es=0.00005;
		double tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7;
		double d1,d2,d3;
		double pi=3.141592653589793;		
		x = xMin + ( (double)xPixel * pixelSize);
		y = yMax - ( (double)yPixel * pixelSize);
		x0=juliaX;
		y0=juliaY;
		//Complex c=new Complex(x0,y0);
		//Complex z=new Complex(1000,0);
		switch(opt){
		case 5:
			p=1-4*x0;
			q=-4*y0;
			r=Math.sqrt(Math.sqrt(p*p+q*q));
			if(p==0) 
				if(q>=0) 
					a=pi/2; 
				else 
					a=-pi/2;
			else
				{
				a=Math.atan(q/p);
				if(p<0) if(q>0) a=a+pi; else a=a-pi;
				}
			a=a/2;
			b=a+pi;
			rx1=-0.5+r*Math.cos(a)/2;
			ry1=r*Math.sin(a)/2;
			rx2=-0.5+r*Math.cos(b)/2;
			ry2=r*Math.sin(b)/2;			
		}
		for (iterationNr=0; iterationNr<maxIterations; iterationNr++) {
			// z^2 + c
			//newx = (x*x) - (y*y) + x0;
			//newy = (2 * x * y) + y0;
			switch(opt){
			case 2:
				//z= new Complex(15*z.real(),15*z.imag());
				//z=(z.zeta()).plus(c);
				//newx=z.real();
				//newy=z.imag();
				nx = Math.cosh(y)*Math.cos(x);
				ny = -Math.sinh(y)*Math.sin(x);
				newx=x0*nx-y0*ny;
				newy=x0*ny+y0*nx;
				break;
			case 3:
				newx=x*x*x-3*x*y*y-x+x0;
				newy=3*x*x*y-y*y*y-y+y0;
				break;
			case 4:
				newx = Math.cosh(y)*Math.cos(x)+x0;
				newy = -Math.sinh(y)*Math.sin(x)+y0;
				break;
			case 5:
				tmp1=x*x;
				tmp2=y*y;
				tmp3=x*(2*tmp1-6*tmp2)+x0;
				tmp4=y*(6*tmp1-2*tmp2)+y0;
				tmp5=3*tmp1-3*tmp2+x0-1;
				tmp6=6*x*y+y0;
				tmp7=tmp5*tmp5+tmp6*tmp6;
				newx=(tmp3*tmp5+tmp4*tmp6)/tmp7;
				newy=(tmp4*tmp5-tmp3*tmp6)/tmp7;				
				break;
			case 6:
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				tmp3=x*tmp1-y*tmp2;
				tmp4=y*tmp1+x*tmp2;
				tmp3=tmp3-4*tmp1;
				tmp4=tmp4-4*tmp2;
				tmp5=1-4*x;
				tmp6=-4*y;
				tmp7=tmp5*tmp5+tmp6*tmp6;
				nx=(tmp3*tmp5+tmp4*tmp6)/tmp7;
				ny=(tmp4*tmp5-tmp3*tmp6)/tmp7;
				newx=x0*nx-y0*ny;
				newy=x0*ny+y0*nx;
				break;
			case 7:
				tmp1=Math.exp(x);
				tmp2=tmp1*Math.cos(y);
				tmp3=tmp1*Math.sin(y);
				tmp4=tmp2*x-tmp3*y;
				tmp5=tmp2*y+tmp3*x;
				newx=tmp4*x0-tmp5*y0;
				newy=tmp4*y0+tmp5*x0;
				break;
			case 8:
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				tmp3=tmp1*x-tmp2*y;
				tmp4=tmp1*y+tmp2*x;
				tmp5=tmp1*tmp1-tmp2*tmp2;
				tmp6=2*tmp1*tmp2;
				newx=tmp5/4-tmp1/2;
				newy=tmp6/4-tmp2/2;
				tmp1=x0*tmp3-y0*tmp4;
				tmp2=y0*tmp3+x0*tmp4;
				newx=newx+tmp1/3;
				newy=newy+tmp2/3;
				tmp1=x0*x-y0*y;
				tmp2=y0*x+x0*y;
				newx=newx-tmp1;
				newy=newy-tmp2;
				break;
			case 9:
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				newx=tmp1*tmp1-tmp2*tmp2;
				newy=2*tmp1*tmp2;
				tmp3=(x*x+y*y);
				tmp3=tmp3*tmp3;
				newx=newx+(x0*tmp1+y0*tmp2)/tmp3;
				newy=newy+(-x0*tmp2+y0*tmp1)/tmp3;
				break;
			default:
			//z= new Complex(15*z.real(),15*z.imag());
			//z=(z.zeta()).plus(c);
			//newx=z.real();
			//newy=z.imag();
			newx=x*x-y*y+x0;
			newy=2*x*y+y0;
		}
				
			x = newx;
			y = newy;
		
			// Well known result: if distance is >2, escapes to infinity...
			switch(opt){
			case 2: case 4:
				if(Math.abs(y)>100) 
					inside = false;
					break;
			case 5:
				d1=(x-1)*(x-1)+y*y;
				d2=(x-rx1)*(x-rx1)+(y-ry1)*(y-ry1);
				d3=(x-rx2)*(x-rx2)+(y-ry2)*(y-ry2);
				if ((d1<es)||(d2<es)||(d3<es)) 
					inside = false;				
				break;
			case 6:
				r=x*x+y*y;
				if ((r>100)||(r<0.01))
					inside = false;
					break;
			case 7:
				if(Math.abs(x)>100) 
					inside = false;
					break;
			default:
				if ((x*x+y*y)>10)
					//if(Math.abs(x)>200) 
					inside = false;
				}
			if(inside==false)
				break;
			}
		
		
		if(inside)
			return colourer.colourInsidePoint();
		else
			return colourer.colourOutsidePoint(iterationNr, maxIterations);
		
	}
	
}
