package uk.ac.ed.inf.mandelbrotmaps;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.preference.PreferenceManager;
import android.util.DisplayMetrics;

public class MandelbrotFractalView extends AbstractFractalView{
	
	public float lastTouchX = 0;
	public float lastTouchY = 0;
	
	public double[] currentJuliaParams = null;//new double[2];
	private float[] pinCoords = new float[2];
	
	Paint outerPinPaint;
	Paint innerPinPaint;
	Paint selectedPinPaint;
	Paint littlePinPaint;
	
	int outerPinAlpha = 150;
	int innerPinAlpha = 150;
	int selectedPinAlpha = 150;
	int littlePinAlpha = 180;
	
	public float smallPinRadius = 5.0f;
	public float largePinRadius = 20.0f;
	
	
	
	public MandelbrotFractalView(Context context, FractalViewSize size) {
		super(context, size);
		
		setColouringScheme(PreferenceManager.getDefaultSharedPreferences(getContext()).getString("MANDELBROT_COLOURS", "RGBWalk")
							, false);
		
		for(int i = 0; i < noOfThreads; i++) {
			renderThreadList.get(i).setName("Mandelbrot thread " + i);
		}
		
		// Set the "maximum iteration" calculation constants
		// Empirically determined values for Mandelbrot set.
		ITERATION_BASE = 1.24;
		ITERATION_CONSTANT_FACTOR = 54;
		
		// Set home area
		homeGraphArea = new MandelbrotJuliaLocation().getMandelbrotGraphArea();
		
		// How deep a zoom do we allow?
		MAXZOOM_LN_PIXEL = -31; // Beyond -31, "double"s break down(!).
		
		//pinColour used to select function (see MandelbrotFractalView, AbstractFractalView and FractalActivity)
		int pinColour = Color.parseColor(PreferenceManager.getDefaultSharedPreferences(getContext()).getString("PIN_COLOUR", "gray"));
		pinColour=0xff888888;
		outerPinPaint = new Paint();
		outerPinPaint.setColor(pinColour);
		outerPinPaint.setAlpha(outerPinAlpha);
		//outerPinPaint.setStyle(Style.STROKE);
		
		innerPinPaint = new Paint();
		innerPinPaint.setColor(pinColour);
		innerPinPaint.setAlpha(innerPinAlpha);
		
		littlePinPaint = new Paint();
		littlePinPaint.setColor(pinColour);
		littlePinPaint.setAlpha(littlePinAlpha);
		littlePinPaint.setStyle(Style.STROKE);
		
		selectedPinPaint = new Paint();
		selectedPinPaint.setColor(pinColour);
		selectedPinPaint.setAlpha(selectedPinAlpha);
	}
		
	
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		
		if(parentActivity.showingLittle && drawPin) {
			if(controlmode != ControlMode.ZOOMING) pinCoords = getPinCoords();
			float[] mappedCoords = new float[2];
			matrix.mapPoints(mappedCoords, pinCoords);
			
			if(fractalViewSize == FractalViewSize.LARGE) {
				canvas.drawCircle(mappedCoords[0], mappedCoords[1], smallPinRadius, innerPinPaint);
				
				//Draw larger outer circle if pin is held down.
				if(!holdingPin)
					canvas.drawCircle(mappedCoords[0], mappedCoords[1], largePinRadius, outerPinPaint);
				else
					canvas.drawCircle(mappedCoords[0], mappedCoords[1], largePinRadius*2, selectedPinPaint);
			}
			else if (fractalViewSize == FractalViewSize.LITTLE) {
				canvas.drawCircle(mappedCoords[0], mappedCoords[1], smallPinRadius, littlePinPaint);
			}
		}
	}
	
	
	/* Runs when the view changes size. 
	 * Sets the size of the pin based on screen size. */
	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		super.onSizeChanged(w, h, oldw, oldh);
	   
		// Show the little view at the start, if allowed.
		if(fractalViewSize == FractalViewSize.LARGE) {
			DisplayMetrics currentDisplayMetrics = new DisplayMetrics();
			parentActivity.getWindowManager().getDefaultDisplay().getMetrics(currentDisplayMetrics);
			
			int dpi = currentDisplayMetrics.densityDpi;
			largePinRadius = dpi/6; 
			smallPinRadius = dpi/30;
		}
	}
	
	
	// Load a location
	void loadLocation(MandelbrotJuliaLocation _mjLocation) {
		//setScaledIterationCount(mjLocation.getMandelbrotContrast());
		if(pixelSizes != null)
			clearPixelSizes();
		//Log.d(TAG, ""+ _mjLocation.getMandelbrotGraphArea()[0]);
		setGraphArea(_mjLocation.getMandelbrotGraphArea(), true);
		currentJuliaParams = _mjLocation.getJuliaParam();
	}
		
	
	int pixelInSet (int xPixel, int yPixel, int maxIterations, int opt) {
		boolean inside = true;
		boolean inside2 = true;
		boolean inside3 = true;
		int iterationNr,iterationNr2,iterationNr3;
		double newx,newy,nx,ny,x,y;
		double a,b,es,p,q,r,rx1,ry1,rx2,ry2;
		rx1=0;
		ry1=0;
		rx2=0;
		ry2=0;
		es=0.00005;
		double tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7;
		double d1,d2,d3;
		double e[]=new double[2];
		double x0=xMin+((double)xPixel*pixelSize);
		double y0=yMax-((double)yPixel*pixelSize ); //TODO This shouldn't be calculated every time
		double pi=3.141592653589793;
		// Start at x0, y0
		x=0;
		y=0;
		//Complex c=new Complex(x0,y0);
		//Complex z=new Complex(1000,0);
		switch(opt){
		case 3:
			x=1/Math.sqrt(3);
			y=0;
			break;
		case 5:
			p=1-4*x0;
			q=-4*y0;
			r=Math.sqrt(Math.sqrt(p*p+q*q));
			if(p==0) 
				if(q>=0) 
					a=pi/2; 
				else 
					a=-pi/2;
			else
				{
				a=Math.atan(q/p);
				if(p<0) if(q>0) a=a+pi; else a=a-pi;
				}
			a=a/2;
			b=a+pi;
			rx1=-0.5+r*Math.cos(a)/2;
			ry1=r*Math.sin(a)/2;
			rx2=-0.5+r*Math.cos(b)/2;
			ry2=r*Math.sin(b)/2;
			break;
		case 6:
			x=(19+Math.sqrt(105))/16;
			y=0;
			break;
		case 7:
			x=-1;
			y=0;
			break;
		case 8:
			x=-1;
			y=0;
			break;
		case 9:
			tmp1=Math.log(0.5*Math.sqrt(x0*x0+y0*y0));
			tmp2=Math.atan2(y0,x0);
			tmp3=Math.exp(tmp1/6);
			x=tmp3*Math.cos(tmp2/6);
			y=tmp3*Math.sin(tmp2/6);
			break;
		}
		//Run iterations over this point
		for (iterationNr=0; iterationNr<maxIterations; iterationNr++) {
			switch(opt){
			case 2:
				//z= new Complex(15*z.real(),15*z.imag());
				//z=(z.gamma()).plus(c);
				//newx=z.real();
				//newy=z.imag();
	
				//c*cos(z)
				nx = Math.cosh(y)*Math.cos(x);
				ny = -Math.sinh(y)*Math.sin(x);
				newx=x0*nx-y0*ny;
				newy=x0*ny+y0*nx;
				break;
			case 3:
				//z^3-z+c
				newx=x*x*x-3*x*y*y-x+x0;
				newy=3*x*x*y-y*y*y-y+y0;
				break;
			case 4:
				//cos(z)+c
				newx = Math.cosh(y)*Math.cos(x)+x0;
				newy = -Math.sinh(y)*Math.sin(x)+y0;
				break;
			case 5:
				//Newton on z^3+(c-1)*z+c (2*z^3+c)/3*z^2+c-1)
				tmp1=x*x;
				tmp2=y*y;
				tmp3=x*(2*tmp1-6*tmp2)+x0;
				tmp4=y*(6*tmp1-2*tmp2)+y0;
				tmp5=3*tmp1-3*tmp2+x0-1;
				tmp6=6*x*y+y0;
				tmp7=tmp5*tmp5+tmp6*tmp6;
				newx=(tmp3*tmp5+tmp4*tmp6)/tmp7;
				newy=(tmp4*tmp5-tmp3*tmp6)/tmp7;				
				break;
			case 6:
				//Herman c*z^2*(z-4)/(1-4*z)
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				tmp3=x*tmp1-y*tmp2;
				tmp4=y*tmp1+x*tmp2;
				tmp3=tmp3-4*tmp1;
				tmp4=tmp4-4*tmp2;
				tmp5=1-4*x;
				tmp6=-4*y;
				tmp7=tmp5*tmp5+tmp6*tmp6;
				nx=(tmp3*tmp5+tmp4*tmp6)/tmp7;
				ny=(tmp4*tmp5-tmp3*tmp6)/tmp7;
				newx=x0*nx-y0*ny;
				newy=x0*ny+y0*nx;
				break;
			case 7:
				//c*z*exp(z)
				tmp1=Math.exp(x);
				tmp2=tmp1*Math.cos(y);
				tmp3=tmp1*Math.sin(y);
				tmp4=tmp2*x-tmp3*y;
				tmp5=tmp2*y+tmp3*x;
				newx=tmp4*x0-tmp5*y0;
				newy=tmp4*y0+tmp5*x0;
				break;
			case 8:
				//Int((z^2-1)*(z+c)
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				tmp3=tmp1*x-tmp2*y;
				tmp4=tmp1*y+tmp2*x;
				tmp5=tmp1*tmp1-tmp2*tmp2;
				tmp6=2*tmp1*tmp2;
				newx=tmp5/4-tmp1/2;
				newy=tmp6/4-tmp2/2;
				tmp1=x0*tmp3-y0*tmp4;
				tmp2=y0*tmp3+x0*tmp4;
				newx=newx+tmp1/3;
				newy=newy+tmp2/3;
				tmp1=x0*x-y0*y;
				tmp2=y0*x+x0*y;
				newx=newx-tmp1;
				newy=newy-tmp2;
				break;
			case 9:
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				newx=tmp1*tmp1-tmp2*tmp2;
				newy=2*tmp1*tmp2;
				tmp3=(x*x+y*y);
				tmp3=tmp3*tmp3;
				newx=newx+(x0*tmp1+y0*tmp2)/tmp3;
				newy=newy+(-x0*tmp2+y0*tmp1)/tmp3;
				break;
			default:
				//z= new Complex(15*z.real(),15*z.imag());
				//z=(z.zeta()).plus(c);
				//newx=z.real();
				//newy=z.imag();
				newx=x*x-y*y+x0;
				newy=2*x*y+y0;
				}				
			x = newx;
			y = newy;	
			// Well known result: if distance is >2, escapes to infinity...
			switch(opt){
			case 2: case 4:
				if(Math.abs(y)>100) 
					inside = false;
					break;
			case 5:
				d1=(x-1)*(x-1)+y*y;
				d2=(x-rx1)*(x-rx1)+(y-ry1)*(y-ry1);
				d3=(x-rx2)*(x-rx2)+(y-ry2)*(y-ry2);
				if ((d1<es)||(d2<es)||(d3<es)) 
					inside = false;				
				break;
			case 6:
				r=x*x+y*y;
				if ((r>100)||(r<0.01))
					inside = false;
					break;
			case 7:
				if(Math.abs(x)>100) 
					inside = false;
					break;
			default:
				if ((x*x+y*y)>10)
				//if(Math.abs(x)>200) 
					inside = false;
				}
			if(inside==false)
				break;
			}
			y=0;				
			if ((opt==3)||(opt==4)||(opt==6)||(opt==8)){
				if(opt==3)
					x=-1/Math.sqrt(3);
				if(opt==4)
					x=3.14159;
				if(opt==6)
					x=(19-Math.sqrt(105))/16;
				if (opt==8)
				{
					x=1;
					y=0;
				}
			for (iterationNr2=0; iterationNr2<maxIterations; iterationNr2++) {
				switch(opt){
				case 3:
					newx=x*x*x-3*x*y*y-x+x0;
					newy=3*x*x*y-y*y*y-y+y0;
					break;
				case 4:
					newx=Math.cosh(y)*Math.cos(x)+x0;
					newy=-Math.sinh(y)*Math.sin(x)+y0;
					break;
				case 6:
					tmp1=x*x-y*y;
					tmp2=2*x*y;
					tmp3=x*tmp1-y*tmp2;
					tmp4=y*tmp1+x*tmp2;
					tmp3=tmp3-4*tmp1;
					tmp4=tmp4-4*tmp2;
					tmp5=1-4*x;
					tmp6=-4*y;
					tmp7=tmp5*tmp5+tmp6*tmp6;
					nx=(tmp3*tmp5+tmp4*tmp6)/tmp7;
					ny=(tmp4*tmp5-tmp3*tmp6)/tmp7;
					newx=x0*nx-y0*ny;
					newy=x0*ny+y0*nx;
					break;
				case 8:
					tmp1=x*x-y*y;
					tmp2=2*x*y;
					tmp3=tmp1*x-tmp2*y;
					tmp4=tmp1*y+tmp2*x;
					tmp5=tmp1*tmp1-tmp2*tmp2;
					tmp6=2*tmp1*tmp2;
					newx=tmp5/4-tmp1/2;
					newy=tmp6/4-tmp2/2;
					tmp1=x0*tmp3-y0*tmp4;
					tmp2=y0*tmp3+x0*tmp4;
					newx=newx+tmp1/3;
					newy=newy+tmp2/3;
					tmp1=x0*x-y0*y;
					tmp2=y0*x+x0*y;
					newx=newx-tmp1;
					newy=newy-tmp2;
					break;
			default:
					newx=x*x-y*y+x0;
					newy=2*x*y+y0;
				}				
				x = newx;
				y = newy;
				switch(opt){
				case 2: case 4:
					if(Math.abs(y)>10) 
						inside2 = false;
						break;
				case 6:
					r=x*x+y*y;
					if ((r>100)||(r<0.01))
						inside2 = false;
						break;
				default:
					if ((x*x+y*y)>10)
						inside2 = false;
				}
				if(inside2==false)
					break;
			}
			if (opt==8){
				x=-x0;
				y=-y0;
				}
			for (iterationNr3=0; iterationNr3<maxIterations; iterationNr3++) {
				tmp1=x*x-y*y;
				tmp2=2*x*y;
				tmp3=tmp1*x-tmp2*y;
				tmp4=tmp1*y+tmp2*x;
				tmp5=tmp1*tmp1-tmp2*tmp2;
				tmp6=2*tmp1*tmp2;
				newx=tmp5/4-tmp1/2;
				newy=tmp6/4-tmp2/2;
				tmp1=x0*tmp3-y0*tmp4;
				tmp2=y0*tmp3+x0*tmp4;
				newx=newx+tmp1/3;
				newy=newy+tmp2/3;
				tmp1=x0*x-y0*y;
				tmp2=y0*x+x0*y;
				newx=newx-tmp1;
				newy=newy-tmp2;
				x = newx;
				y = newy;
				if ((x*x+y*y)>10)
					{
					inside3 = false;
					break;
					}
		}			
		if ((inside==true)&&(inside2==true))
			inside=true;
		else {
			inside=false;
			if ((opt==3)||(opt==4)||(opt==6))
				iterationNr=Math.round((iterationNr+iterationNr2)/2);			
			if (opt==8)
				iterationNr=Math.round((iterationNr+iterationNr2+iterationNr3)/3);			
			}
	}		
		if(inside)
			return colourer.colourInsidePoint();
		else
			return colourer.colourOutsidePoint(iterationNr, maxIterations);
	}
	
	
	public double[] getJuliaParams(float touchX, float touchY)
	{
		lastTouchX = touchX;
		lastTouchY = touchY;
		
		double[] mandelbrotGraphArea = graphArea;
		double pixelSize = getPixelSize();
	
		double[] juliaParams = new double[2];
		
		// Touch position, on the complex plane (translated from pixels)
		juliaParams[0] = mandelbrotGraphArea[0] + ( (double)touchX * pixelSize );
		juliaParams[1] = mandelbrotGraphArea[1] - ( (double)touchY * pixelSize );
		
		currentJuliaParams = juliaParams;
		
		return juliaParams;
	}
	
	
	public float[] getPinCoords() {		
		float[] pinCoords = new float[2];
		double pixelSize = getPixelSize();
		
		if (fractalViewSize == FractalViewSize.LITTLE) {
			currentJuliaParams = ((JuliaFractalView)parentActivity.fractalView).getJuliaParam();
		}

		pinCoords[0] = (float) ((currentJuliaParams[0] - graphArea[0]) / pixelSize);
		pinCoords[1] = (float) (-(currentJuliaParams[1] - graphArea[1]) / pixelSize);
		
		return pinCoords;
	}
	
	
	public void setPinColour(int newColour) {
		outerPinPaint.setColor(newColour);
		selectedPinPaint.setColor(newColour);
		innerPinPaint.setColor(newColour);
		littlePinPaint.setColor(newColour);
		
		// This somehow resets the alphas as well, so reset those.
		outerPinPaint.setAlpha(outerPinAlpha);
		innerPinPaint.setAlpha(innerPinAlpha);
		littlePinPaint.setAlpha(littlePinAlpha);		
		selectedPinPaint.setAlpha(selectedPinAlpha);
		
		invalidate();
	}
}
